/**
 * Desc inheritances
 * @function descInheritance
 * @param {object} configuration
 */

'use strict'

const pathtree = require('pathtree')
const arrayreduce = require('arrayreduce')
const path = require('path')

const separator = '~'

/** @lends descInheritance */
function descInheritance (configuration) {
  let tree = pathtree.create({ separator })

  let $proto = [].concat(configuration.$proto || [])
  return tree.toString(
    $proto.map(_descOfApemanfile).reduce(arrayreduce.arrayConcat(), []),
    _nameOfApemanfile(configuration)
  )
}

function _nameOfApemanfile (configuration) {
  let { $name } = configuration
  if ($name) {
    return $name
  }

  let { $pkg, $src, $cwd } = configuration

  let name = ($pkg && $pkg.name) || ($src && path.relative(process.cwd(), $src)) || ($cwd && path.resolve($cwd))
  let version = $pkg && $pkg.version
  if (version) {
    name = [ name, version ].join('@')
  }
  return name
}

function _descOfApemanfile (apemanfile) {
  if (!apemanfile) {
    return []
  }
  let name = _nameOfApemanfile(apemanfile)
  let $proto = [].concat(apemanfile.$proto || [])
  return [ name ].concat(
    $proto.map(($proto) =>
      _descOfApemanfile($proto).map((desc) =>
        [ name, desc ].join(separator)
      )
    ).reduce(arrayreduce.arrayConcat(), [])
  )
}

module.exports = descInheritance
